@@ -26,16 +26,16 @@ use std::rc::Rc; // type forms are kinda bulky
 /// This is safe if directly inside a `Node` that was just freshened.
 /// (TODO: think about what "just" means here. It's super-subtle!)
 pub fn strip_ee(a: &Ast) -> &Ast {
-    match *a {
-        ExtendEnv(ref body, _) => (&**body),
-        ExtendEnvPhaseless(ref body, _) => (&**body),
+    match a.c() {
+        ExtendEnv(body, _) => (&*body),
+        ExtendEnvPhaseless(body, _) => (&*body),
         _ => icp!("Not an EE"),
     }
 }
 
 pub fn strip_ql(a: &Ast) -> &Ast {
-    match a {
-        QuoteLess(ref body, _) => &**body,
+    match a.c() {
+        QuoteLess(body, _) => &*body,
         _ => icp!("Not an unquote"),
     }
 }
@@ -123,7 +123,7 @@ fn type_match(part_types: LazyWalkReses<SynthTy>) -> TypeResult {
         None => {
             // TODO #2: this isn't anywhere near exhaustive
             ty_err!(NonExhaustiveMatch(part_types.get_res(n("scrutinee")).unwrap())
-                at Trivial /* TODO */)
+                at ast!((trivial)) /* TODO */)
         }
         Some(ty_res) => Ok(ty_res),
     }
@@ -216,7 +216,7 @@ fn type_unfold(part_types: LazyWalkReses<SynthTy>) -> TypeResult {
     mu_parts;
     {
         // This acts like the `mu` was never there (and hiding the binding)
-        if let ExtendEnv(ref body, _) = *mu_parts.get_leaf_or_panic(&n("body")) {
+        if let ExtendEnv(body, _) = mu_parts.get_leaf_or_panic(&n("body")).c() {
             synth_type(body, part_types.env)
         } else { icp!("no protection to remove!"); }
     })
@@ -232,7 +232,7 @@ fn type_fold(part_types: LazyWalkReses<SynthTy>) -> TypeResult {
     mu_parts;
     {
         // This acts like the `mu` was never there (and hiding the binding)
-        if let ExtendEnv(ref body, _) = *mu_parts.get_leaf_or_panic(&n("body")) {
+        if let ExtendEnv(ref body, _) = mu_parts.get_leaf_or_panic(&n("body")).c() {
             synth_type(body, part_types.env.clone())?
         } else {
             icp!("no protection to remove!");
@@ -255,7 +255,11 @@ fn type__forall_expr(part_types: LazyWalkReses<SynthTy>) -> TypeResult {
 
 /// This is the Unseemly language.
 pub fn make_core_syn_env() -> SynEnv {
-    color_backtrace::install(); // HACK: this is around the first thing that happens in any test.
+    // HACK: this is around the first thing that happens, so install a panic hook.
+    #[cfg(not(target_arch = "wasm32"))]
+    color_backtrace::install(); // Nice backtraces
+    #[cfg(target_arch = "wasm32")]
+    std::panic::set_hook(Box::new(console_error_panic_hook::hook)); // Backtrace to console
 
     let ctf: SynEnv = get_core_types();
     let cmf: SynEnv = crate::core_macro_forms::make_core_macro_forms();
@@ -376,7 +380,7 @@ pub fn make_core_syn_env() -> SynEnv {
                             return Ok(res);
                         }
                         ty_err!(NonexistentEnumArm(arm_name.to_name(),
-                            Trivial) /* TODO `LazyWalkReses` needs more information */
+                            ast!((trivial))) /* TODO `LazyWalkReses` needs more information */
                             at arm_name.clone())
                 }
             )),
@@ -503,9 +507,11 @@ pub fn make_core_syn_env() -> SynEnv {
     let reserved_names = vec![
         n("forall"),
         n("mu_type"),
-        n("Int"),
         n("Ident"),
+        n("Int"),
+        n("Nat"),
         n("Float"),
+        n("String"),
         n("match"),
         n("enum"),
         n("struct"),
@@ -529,14 +535,23 @@ pub fn make_core_syn_env() -> SynEnv {
         "DefaultSeparator" => (scan r"(\s*)"),
         "DefaultAtom" => (common (reserved_by_name_vec (call "DefaultWord"), reserved_names)),
         "DefaultWord" => (common (pick [(call "DefaultSeparator"),
-            (named "name", (scan r"(\p{Letter}(?:\p{Letter}|\p{Number}|[_?])*)"))],
+            (named "name", (scan_cat r"(\p{Letter}(?:\p{Letter}|\p{Number}|[_?])*)",
+                                     "variable"))],
                 "name")),
         // TODO: come up with more normal tokenization rules.
         // HACK: it's really confusing to weld semicolon and colon onto brackets, so exempt them.
+        "OpenDelim" =>
+            (common (pick [(call "DefaultSeparator"),
+                (named "tok",
+                    (scan_cat r"([^\[\]\(\)\{\}\s;:]*[\[\(\{])", "paren.lparen"))], "tok")),
+        "CloseDelim" =>
+            (common (pick [(call "DefaultSeparator"),
+                (named "tok",
+                    (scan_cat r"([\]\)\}][^\[\]\(\)\{\}\s;:]*)", "paren.rparen"))], "tok")),
         "DefaultToken" =>
            (common (pick [(call "DefaultSeparator"),
                    (named "tok",
-                       (scan r"([\]\)\}][^\[\]\(\)\{\}\s;:]*|[^\[\]\(\)\{\}\s;:]*[\[\(\{]|[^\[\]\(\)\{\}\s]+)"))],
+                       (scan r"([^\[\]\(\)\{\}\s]+)"))],
             "tok"))
     )
     .set_assoc(&ctf) // throw in types!
