@@ -1,5 +1,6 @@
 use crate::{
-    ast::{Ast, Atom, Node},
+    ast,
+    ast::Ast,
     ast_walk::{
         LazyWalkReses,
         WalkRule::{Body, LiteralLike, NotWalked},
@@ -120,14 +121,14 @@ fn macro_type(forall_ty_vars: &[Name], arguments: Vec<(Name, Ast)>, output: Ast)
         // TODO: this can go wrong if a macro-defining macro collides two term names.
         // Fixing this probably requires rethinking how "component_name" works.
         // Perhaps not using structs at all might also work.
-        components.push(mbe!("component_name" => (, Atom(k.unhygienic_orig())),
+        components.push(mbe!("component_name" => (, ast!(k.unhygienic_orig())),
                              "component" => (, v.to_ast())));
     }
-    let argument_struct = Node(
+    let argument_struct = raw_ast!(Node(
         crate::core_forms::find_core_form("Type", "struct"),
         crate::util::mbe::EnvMBE::new_from_anon_repeat(components),
-        ExportBeta::Nothing,
-    );
+        ExportBeta::Nothing
+    ));
     let mac_fn = u!({Type fn : [(, argument_struct)] (, output.to_ast())});
 
     if forall_ty_vars.is_empty() {
@@ -135,7 +136,7 @@ fn macro_type(forall_ty_vars: &[Name], arguments: Vec<(Name, Ast)>, output: Ast)
     } else {
         ast!({"Type" "forall_type" :
             "body" => (import [* [forall "param"]] (, mac_fn)),
-            "param" => (,seq forall_ty_vars.iter().map(|n| { Atom(*n) }).collect::<Vec<_>>())
+            "param" => (,seq forall_ty_vars.iter().map(|n: &Name| { ast!(*n) }).collect::<Vec<_>>())
         })
     }
 }
@@ -188,7 +189,7 @@ fn type_macro_invocation(
     )
     .map_err(|e| crate::util::err::sp(e, parts.this_ast.clone()))?;
 
-    // TODO: I think `Assoc` should implement `From<Vec<(K,V)>>`, maybe?
+    // TODO: `Assoc` should implement `From<Vec<(K,V)>>`
     let mut res = Assoc::new();
     for (k, v) in q_arguments {
         res = res.set(k, v.clone())
@@ -222,9 +223,7 @@ pub fn macro_invocation(
         name: n("macro_invocation"), // TODO: maybe generate a fresh name?
         grammar: Rc::new(form_pat!([
             // `type_macro_invocation` expects "macro_name" to be set
-            (named "macro_name", (anyways (,
-                Ast::VariableReference(macro_name)
-            ))),
+            (named "macro_name", (anyways (vr macro_name))),
             // Capture this here so that its environmental names get freshened properly.
             // Need to store this one phase unquoted.
             (named "impl", (-- 1 (anyways (,impl_body)))),
@@ -304,8 +303,8 @@ pub fn macro_invocation(
                             // Nuke all binding, since we're abandoning its context.
                             // The user will† deposit this syntax inside a replacement binding form.
                             // (†still not enforced until issue #31 is fixed)
-                            while let Ast::ExtendEnv(ref body, _)
-                            | Ast::ExtendEnvPhaseless(ref body, _) = a
+                            while let ast::ExtendEnv(ref body, _)
+                            | ast::ExtendEnvPhaseless(ref body, _) = a.c()
                             {
                                 a = &*body;
                             }
@@ -334,8 +333,8 @@ pub fn macro_invocation(
 fn repeated_type(t: &Ast, env: &Assoc<Name, Ast>) -> Result<Ast, crate::ty::TypeError> {
     let mut drivers = vec![];
     for v in t.free_vrs() {
-        if env.find(&v) == Some(&Ast::VariableReference(v)) {
-            drivers.push(Ast::VariableReference(v));
+        if env.find(&v).map(|a| a.c()) == Some(&ast::VariableReference(v)) {
+            drivers.push(env.find_or_panic(&v).clone());
         }
     }
 
@@ -424,7 +423,7 @@ pub fn make_core_macro_forms() -> SynEnv {
     // Most of "Syntax" is a negative walk (because it produces an environment),
     //  but lacking a `negative_ret_val`.
     let grammar_grammar = forms_to_form_pat_export![
-        syntax_syntax!( ( (delim "anyways,{", "{", (named "body", (call "Expr"))) ) Anyways (
+        syntax_syntax!( ( (delim "anyways{", "{", (named "body", (call "Expr"))) ) Anyways (
             body => Ast::reflect(&body)
         )) => ["body"],
         // HACK: expanding to `'[Expr| capture_language]'` doesn't do what you want, so do this:
@@ -434,11 +433,26 @@ pub fn make_core_macro_forms() -> SynEnv {
             type_compare: Both(NotWalked, NotWalked),
             synth_type: Negative(cust_rc_box!(|_| Ok(Assoc::new()))),
             eval: Positive(cust_rc_box!(move |_| {
-                Ok(FormPat::Scope(capture_language_form.clone(), crate::beta::ExportBeta::Nothing).reify())
+                Ok(FormPat::Scope(capture_language_form.clone(),
+                                  crate::beta::ExportBeta::Nothing).reify())
             })),
             quasiquote: Both(LiteralLike, LiteralLike)
         }) => [],
         syntax_syntax!( ((lit "impossible")) Impossible ) => [],
+        syntax_syntax!( ([(named "body", (call "Syntax")), (lit "reserving"),
+                (star (named "words", (scan r"\s*'((?:[^'\\]|\\'|\\\\)*)'")))
+            ]) Reserved {
+        |parts| {
+            parts.get_res(n("body"))
+        }
+        } {
+        |parts| {
+            Ok(Reserved(Rc::new(FormPat::reflect(&parts.get_res(n("body"))?)),
+                parts.get_rep_term(n("words")).iter().map(Ast::to_name).collect::<Vec<_>>()
+            ).reify())
+            }
+        }) => ["body"],
+
         syntax_syntax!( (  // TODO: this might have to be both positive and negative
             [(lit "lit"), (named "body", (call "Syntax")),
              // Allow \\ and \' as escapes:
@@ -458,10 +472,6 @@ pub fn make_core_macro_forms() -> SynEnv {
         syntax_syntax!( ([(lit "vr"), (delim "(", "(", (named "body", (call "Syntax")))]) VarRef (
             body => Rc::new(FormPat::reflect(&body))
         )) => [],
-        syntax_syntax!( ([(lit "common"), (delim "(", "(", (named "body", (call "Syntax")))]) Common
-        (
-            body => Rc::new(FormPat::reflect(&body))
-        )) => [],
         // TODO: split out a separate SyntaxSeq, so that we can get rid of the [ ] delimiters
         syntax_syntax!( ( (delim "[", "[", (star (named "elt", (call "Syntax"))))) Seq {
             |parts| {
@@ -498,19 +508,6 @@ pub fn make_core_macro_forms() -> SynEnv {
                 Ok(Plus(Rc::new(FormPat::reflect(&parts.get_res(n("body"))?))).reify())
             }
         }) => ["body"],
-        syntax_syntax!( ([(named "body", (call "Syntax")), (lit "reserving"),
-                          (star (named "words", (scan r"\s*'((?:[^'\\]|\\'|\\\\)*)'")))
-                         ]) Reserved {
-            |parts| {
-                parts.get_res(n("body"))
-            }
-        } {
-            |parts| {
-                Ok(Reserved(Rc::new(FormPat::reflect(&parts.get_res(n("body"))?)),
-                   parts.get_rep_term(n("words")).iter().map(Ast::to_name).collect::<Vec<_>>()
-                ).reify())
-            }
-        }) => ["body"],
         // TODO: support seprators, and add a separator here
         syntax_syntax!( ( (delim "alt[", "[", (star [(named "elt", (call "Syntax"))]))) Alt {
             |parts| {
@@ -538,7 +535,7 @@ pub fn make_core_macro_forms() -> SynEnv {
         //     This has to have the same named parts as `unquote`, because it reuses its typechecker
         //     But the type walk (as an overall quotation and locally) is always negative.
         syntax_syntax!( ([(named "part_name", atom), (lit ":="),
-                          (lit "("), (named "body", (call "Syntax")), (lit ")")])
+                          (delim "(", "(", (named "body", (call "Syntax")))])
         Named {
             |parts| {
                 let binder = parts.get_term(n("part_name")).to_name();
@@ -589,10 +586,11 @@ pub fn make_core_macro_forms() -> SynEnv {
         Rc::new(Form {
             name: n("scan"),
             grammar: Rc::new(form_pat!(
-                (named "pat", (pick
-                    [(call "DefaultSeparator"), (named "pat", (scan r"/((?:[^/\\]|\\.)*)/"))],
-                    "pat"))
-            )),
+                [(call "DefaultSeparator"),
+                 (named "pat", (scan_cat r"/((?:[^/\\]|\\.)*)/", "string.regexp")),
+                 (alt [], [
+                     (lit "as"), (call "DefaultSeparator"),
+                     (named "category", (scan r"((?:\p{Letter}|[-.])*)"))])])),
             type_compare: Both(NotWalked,NotWalked), // Not a type
             synth_type: Both(
                 cust_rc_box!(|_| { Ok(ast!({"Type" "Ident" :})) }),
@@ -600,10 +598,27 @@ pub fn make_core_macro_forms() -> SynEnv {
             eval: Positive(cust_rc_box!(|parts| {
                 let regex = parts.get_term(n("pat")).to_name().orig_sp()
                     .replace(r#"\/"#, r#"/"#);
-                Ok(crate::grammar::new_scan(&regex).reify())
+                Ok(crate::grammar::new_scan(
+                    &regex,
+                    parts.maybe_get_term(n("category")).map(|a| a.to_name().orig_sp())
+                ).reify())
             })),
             quasiquote: Both(LiteralLike, LiteralLike)
         }) => [],
+        // `Common` can be positive or negative (may be under a `Named`)
+        Rc::new(Form {
+            name: n("common"),
+            grammar: Rc::new(form_pat!(
+                [(lit "common"), (delim "(", "(", (named "body", (call "Syntax")))])),
+            type_compare: Both(NotWalked,NotWalked), // Not a type
+            synth_type: Both(
+                cust_rc_box!(|parts| { parts.get_res(n("body")) }),
+                cust_rc_box!(|parts| { parts.get_res(n("body")) })),
+            eval: Positive(cust_rc_box!(|parts| {
+                Ok(Common(Rc::new(FormPat::reflect(&parts.get_res(n("body"))?))).reify())
+            })),
+            quasiquote: Both(LiteralLike, LiteralLike)
+        }) => ["body"],
         // `Import` is positive (has to be under a `Named`)
         Rc::new(Form {
             name: n("import"),
@@ -705,20 +720,27 @@ pub fn make_core_macro_forms() -> SynEnv {
         "Beta" => Rc::new(beta_grammar))
 }
 
+thread_local! {
+    /// Per-file highlighting keeps track of the successive languages,
+    ///  and looks for the syntax extensions that trigger them:
+    pub static syn_envs__for__highlighting: std::cell::RefCell<Vec<(Ast, SynEnv)>>
+        = std::cell::RefCell::new(vec![]);
+}
+
 pub fn extend_syntax() -> Rc<Form> {
     use crate::earley::ParseContext;
     let perform_extension = move |pc: ParseContext, extension_info: Ast| -> ParseContext {
         let bnf_parts =
             // TODO: getting a `Shape` (the second element is the `(lit "in")`) must be a parser bug
-            extract!((&extension_info) Ast::Shape = (ref subs) =>
-                extract!((&subs[0]) Ast::IncompleteNode = (ref parts) => parts));
+            extract!((extension_info.c()) ast::Shape = (ref subs) =>
+                extract!((subs[0].c()) ast::IncompleteNode = (ref parts) => parts));
 
         let nts: Vec<Name> =
             bnf_parts.get_rep_leaf_or_panic(n("nt")).iter().map(|a| a.to_name()).collect();
         let ops: Vec<bool> = bnf_parts
             .get_rep_leaf_or_panic(n("operator"))
             .iter()
-            .map(|a| a == &&Ast::Atom(n("::=also")))
+            .map(|a| a.to_name() == n("::=also"))
             .collect();
         let rhses: Vec<&Ast> = bnf_parts.get_rep_leaf_or_panic(n("rhs"));
 
@@ -738,6 +760,11 @@ pub fn extend_syntax() -> Rc<Form> {
             )
         }
 
+        syn_envs__for__highlighting.with(|envs| {
+            // For per-file highlighting:
+            envs.borrow_mut().push((extension_info, syn_env.clone()));
+        });
+
         ParseContext { grammar: syn_env, type_ctxt: pc.type_ctxt, eval_ctxt: pc.eval_ctxt }
     };
 
@@ -774,12 +801,14 @@ pub fn extend_syntax() -> Rc<Form> {
 fn formpat_reflection() {
     use crate::{core_forms::find_form, runtime::eval::eval_top};
     let macro_forms = make_core_macro_forms()
-        .set(n("DefaultToken"), Rc::new(crate::grammar::new_scan(r"\s*(\S+)")))
+        .set(n("DefaultToken"), Rc::new(crate::grammar::new_scan(r"\s*(\S+)", None)))
+        .set(n("OpenDelim"), Rc::new(crate::grammar::new_scan(r"\s*(\S+)", None)))
+        .set(n("CloseDelim"), Rc::new(crate::grammar::new_scan(r"\s*(\S+)", None)))
         .set(n("DefaultAtom"), Rc::new(FormPat::Call(n("DefaultToken"))))
         .set(n("AtomNotInPat"), Rc::new(FormPat::Call(n("DefaultToken"))))
         .set(n("DefaultReference"), Rc::new(VarRef(Rc::new(FormPat::Call(n("DefaultToken"))))))
         .set(n("Type"), Rc::new(FormPat::Call(n("DefaultReference"))))
-        .set(n("DefaultSeparator"), Rc::new(crate::grammar::new_scan(r"(\s*)")));
+        .set(n("DefaultSeparator"), Rc::new(crate::grammar::new_scan(r"(\s*)", None)));
 
     fn syntax_to_form_pat(a: Ast) -> FormPat { FormPat::reflect(&eval_top(&a).unwrap()) }
 
@@ -805,7 +834,7 @@ fn formpat_reflection() {
         )
     };
 
-    assert_eq!(string_to_form_pat(r"/\s*(\S+)/"), crate::grammar::new_scan(r"\s*(\S+)"));
+    assert_eq!(string_to_form_pat(r"/\s*(\S+)/"), crate::grammar::new_scan(r"\s*(\S+)", None));
     assert_eq!(string_to_form_pat(r"lit /\s*(\S+)/ = 'x'"), form_pat!((lit_aat "x")));
     assert_eq!(
         string_to_form_pat(r"[ lit /\s*(\S+)/ = 'write_this' ,{ Expr < Int > }, <-- nothing ]"),
@@ -1058,9 +1087,9 @@ fn define_and_parse_macros() {
         crate::core_forms::outermost__parse_context(),
         "extend_syntax
              Expr ::=also forall T . '{
-                 [ lit ,{ DefaultToken }, = '['
+                 [ lit ,{ OpenDelim }, = '['
                      body := ( ,{ Expr< Int > }, )
-                   lit ,{ DefaultToken }, = ']'
+                   lit ,{ CloseDelim }, = ']'
                  ]
              }' add_one__macro -> .{ '[ Expr | (plus one ,[body], ) ]' }. ;
         in [ [ [ one ] ] ]",
