@@ -48,10 +48,7 @@ use crate::{
         WalkRule::{self, *},
     },
     form::{simple_form, BiDiWR, Both, Form, Positive},
-    grammar::{
-        FormPat::{self, *},
-        SynEnv,
-    },
+    grammar::{FormPat, SynEnv},
     name::*,
     ty::{synth_type, SynthTy, TyErr},
     ty_compare::{Canonicalize, Subtype},
@@ -102,7 +99,7 @@ thread_local! {
 }
 
 pub fn get__primitive_type(called: Name) -> Ast {
-    ast!({primitive_type.with(|p_t| p_t.clone()) ; "name" => (, Atom(called))})
+    ast!({primitive_type.with(|p_t| p_t.clone()) ; "name" => (at called)})
 }
 
 fn is_primitive(form: &Rc<Form>) -> bool { form == &primitive_type.with(|p_t| p_t.clone()) }
@@ -282,8 +279,9 @@ fn make_core_syn_env_types() -> SynEnv {
                 // Apply the Amber rule; assume the `mu`ed names are subtypes to subtype the bodies
                 let mut amber_environment = mu_parts.env.clone();
                 for (&ee_r, ee_l) in r_params.iter().zip(l_params.iter()) {
-                    let (p_r, p_l) = if let (ExtendEnv(r, _), ExtendEnv(l, _)) = (ee_r, ee_l) {
-                        (&**r, &**l)
+                    let (p_r, p_l) = if let (ExtendEnv(r, _), ExtendEnv(l, _))
+                            = (ee_r.c(), ee_l.c()) {
+                        (&*r, &*l)
                     } else {
                         icp!("ill-formed mu_type")
                     };
@@ -328,17 +326,17 @@ fn make_core_syn_env_types() -> SynEnv {
     // We restrict the LHS to being a name, because that's "normal". Should we?
     let type_apply = type_defn_complex(
         "type_apply",
-        // TODO: invoking `scan` directly to ad-hoc tokenize ... isn't ideal.
+        // TODO: this ad-hoc rule to allow `A<B>` without spaces ... isn't ideal.
         form_pat!([(named "type_rator", (call "Type")),
-                   (call "DefaultSeparator"), (scan r"(<)"),
+                   (call "DefaultSeparator"), (lit_tok (scan "(<)"), "<"),
                    (star (named "arg", (call "Type"))),
-                   (call "DefaultSeparator"), (scan r"(>)")]),
+                   (call "DefaultSeparator"), (lit_tok (scan "(>)"), ">")]),
         // TODO: shouldn't it be "args"?
         cust_rc_box!(move |tapp_parts| {
             use crate::util::mbe::EnvMBE;
             let arg_res = tapp_parts.get_rep_res(n("arg"))?;
             let rator_res = tapp_parts.get_res(n("type_rator"))?;
-            match rator_res {
+            match rator_res.c() {
                 VariableReference(rator_vr) => {
                     // e.g. `X<int, Y>` underneath `mu X. ...`
 
@@ -347,7 +345,7 @@ fn make_core_syn_env_types() -> SynEnv {
                     //  we wish to avoid aliasing problems at the type level.
                     // In System F, this is avoided by performing capture-avoiding substitution.
                     let mut new__tapp_parts = EnvMBE::new_from_leaves(
-                        assoc_n!("type_rator" => VariableReference(rator_vr)),
+                        assoc_n!("type_rator" => ast!((vr *rator_vr))),
                     );
 
                     let mut args = vec![];
@@ -358,8 +356,8 @@ fn make_core_syn_env_types() -> SynEnv {
                     }
                     new__tapp_parts.add_anon_repeat(args);
 
-                    if let Node(ref f, _, ref exp) = tapp_parts.this_ast {
-                        Ok(Node(/* forall */ f.clone(), new__tapp_parts, exp.clone()))
+                    if let Node(ref f, _, ref exp) = tapp_parts.this_ast.c() {
+                        Ok(raw_ast!(Node(/* forall */ f.clone(), new__tapp_parts, exp.clone())))
                     } else {
                         icp!()
                     }
@@ -367,7 +365,7 @@ fn make_core_syn_env_types() -> SynEnv {
                 Node(ref got_f, ref lhs_parts, ref exports) if is_primitive(got_f) => {
                     // Like the above; don't descend into `Expr`
                     let mut new__tapp_parts = EnvMBE::new_from_leaves(assoc_n!("type_rator" =>
-                            Node(got_f.clone(), lhs_parts.clone(), exports.clone())));
+                            raw_ast!(Node(got_f.clone(), lhs_parts.clone(), exports.clone()))));
                     let mut args = vec![];
                     for individual__arg_res in arg_res {
                         args.push(EnvMBE::new_from_leaves(
@@ -376,8 +374,8 @@ fn make_core_syn_env_types() -> SynEnv {
                     }
                     new__tapp_parts.add_anon_repeat(args);
 
-                    if let Node(ref f, _, ref exp) = tapp_parts.this_ast {
-                        Ok(Node(/* forall */ f.clone(), new__tapp_parts, exp.clone()))
+                    if let Node(f, _, exp) = tapp_parts.this_ast.c() {
+                        Ok(raw_ast!(Node(/* forall */ f.clone(), new__tapp_parts, exp.clone())))
                     } else {
                         icp!()
                     }
@@ -410,25 +408,29 @@ fn make_core_syn_env_types() -> SynEnv {
         Both(LiteralLike, LiteralLike),
     );
 
-    assoc_n!("Type" => Rc::new(Biased(Rc::new(forms_to_form_pat![
-        fn_type,
-        // TODO: these should turn into `primitive_type`s in the core type environment.
-        // First, we need a really simple core type environment for testing,
-        //  and then to change all the `uty!({Type Int :})`s into `uty!(Int)`s
-        //  (and `ast!({"Type" "Int" :})`s into `ast!((vr "Int"))`).
-        type_defn("Ident", form_pat!((name_lit "Ident"))),
-        type_defn("Int", form_pat!((name_lit "Int"))),
-        type_defn("Nat", form_pat!((name_lit "Nat"))),
-        type_defn("Float", form_pat!((name_lit "Float"))),
-        type_defn("String", form_pat!((name_lit "String"))),
-        enum_type,
-        struct_type,
-        tuple_type,
-        forall_type,
-        dotdotdot_type,
-        mu_type,
-        type_apply
-        ]), Rc::new(VarRef(Rc::new(Call(n("DefaultAtom"))))))))
+    assoc_n!("Type" => Rc::new(form_pat![
+        // Disambiguate `forall T. Foo<T>` so it doesn't get parsed as `(forall T. Foo)<T>`:
+        (biased
+            (scope forall_type),
+            (alt
+                (scope fn_type),
+                // TODO: these should turn into `primitive_type`s in the core type environment.
+                // First, we need a really simple core type environment for testing,
+                //  and then to change all the `uty!({Type Int :})`s into `uty!(Int)`s
+                //  (and `ast!({"Type" "Int" :})`s into `ast!((vr "Int"))`).
+                (scope type_defn("Ident", form_pat!((name_lit "Ident")))),
+                (scope type_defn("Int", form_pat!((name_lit "Int")))),
+                (scope type_defn("Nat", form_pat!((name_lit "Nat")))),
+                (scope type_defn("Float", form_pat!((name_lit "Float")))),
+                (scope type_defn("String", form_pat!((name_lit "String")))),
+                (scope enum_type),
+                (scope struct_type),
+                (scope tuple_type),
+                (scope dotdotdot_type),
+                (scope mu_type),
+                (scope type_apply),
+                varref))
+    ]))
 }
 
 thread_local! {
