@@ -17,7 +17,8 @@
 // Also, it turns out that implementing an Earley parser goes pretty smoothly. Yay!
 
 use crate::{
-    ast::Ast,
+    ast,
+    ast::{Ast, LocatedAst},
     ast_walk::LazyWalkReses,
     grammar::{
         FormPat::{self, *},
@@ -42,6 +43,9 @@ thread_local! {
     // For parse error reporting: how far have we gotten?
     static best_token: RefCell<(usize, Rc<FormPat>, usize)>
         = RefCell::new((0, Rc::new(Impossible), 0));
+
+    pub static files: RefCell<codespan_reporting::files::SimpleFiles<String, String>>
+        = RefCell::new(codespan_reporting::files::SimpleFiles::new())
 }
 
 fn get_next_id() -> UniqueId {
@@ -88,11 +92,11 @@ impl crate::runtime::reify::Reifiable for ParseContext {
     fn ty_name() -> Name { n("Language") }
 
     fn reify(&self) -> crate::runtime::eval::Value {
-        crate::runtime::eval::Value::Language(Box::new(self.clone()))
+        crate::runtime::eval::Value::ParseContext(Box::new(self.clone()))
     }
 
     fn reflect(v: &crate::runtime::eval::Value) -> ParseContext {
-        extract!((v) crate::runtime::eval::Value::Language = (ref lang)
+        extract!((v) crate::runtime::eval::Value::ParseContext = (ref lang)
             => (**lang).clone())
     }
 }
@@ -107,6 +111,7 @@ impl ParseContext {
     pub fn new_from_grammar(se: SynEnv) -> ParseContext {
         ParseContext {
             grammar: se,
+            // TODO: uh, I think this is unused?
             type_ctxt: LazyWalkReses::<crate::ty::SynthTy>::new_empty(),
             eval_ctxt: LazyWalkReses::<crate::runtime::eval::Eval>::new_empty(),
         }
@@ -129,6 +134,8 @@ pub struct Item {
     // -- Just for error messages --
     /// This rule is too commonplace to be informative in a parse error
     common: bool,
+    /// What file are we in?
+    file_id: usize,
 
     // -- Everything after this line is nonstandard, and is just here as an optimization--
     /// Identity for the purposes of `wanted_by` and `local_parse`
@@ -167,6 +174,7 @@ enum LocalParse {
     NothingYet,
     JustifiedByItemPlanB(UniqueIdRef), // Looking for a better parse, though... (for `Biased`)
     JustifiedByItem(UniqueIdRef),
+    /// This might be more than an atom, but *only* in the rare `Anyways` case.
     ParsedAtom(Ast),
     /// ⊥; contradiction (TMI!)
     Ambiguous(Box<LocalParse>, Box<LocalParse>),
@@ -203,6 +211,7 @@ impl Clone for Item {
             pos: self.pos,
             pc: self.pc.clone(),
             common: self.common,
+            file_id: self.file_id,
             id: get_next_id(),
             done: self.done.clone(),
             local_parse: RefCell::new(LocalParse::NothingYet),
@@ -213,7 +222,12 @@ impl Clone for Item {
 
 /// Progress through the state sets
 // TODO: this ought to produce an Option<ParseError>, not a bool!
-fn create_chart(rule: Rc<FormPat>, pc: ParseContext, toks: &str) -> (UniqueId, Vec<Vec<Item>>) {
+fn create_chart(
+    rule: Rc<FormPat>,
+    pc: ParseContext,
+    toks: &str,
+    file_id: usize,
+) -> (UniqueId, Vec<Vec<Item>>) {
     let toks = toks.trim(); // HACK: tokens don't consume trailing whitespace
     let mut chart: Vec<Vec<Item>> = vec![];
     chart.resize_with(toks.len() + 1, std::default::Default::default);
@@ -226,6 +240,7 @@ fn create_chart(rule: Rc<FormPat>, pc: ParseContext, toks: &str) -> (UniqueId, V
         pos: 0,
         pc: Rc::new(pc),
         common: false,
+        file_id: file_id,
         id: get_next_id(),
         done: RefCell::new(false),
         local_parse: RefCell::new(LocalParse::NothingYet),
@@ -244,9 +259,14 @@ fn create_chart(rule: Rc<FormPat>, pc: ParseContext, toks: &str) -> (UniqueId, V
 }
 
 /// Recognize `rule` in `grammar` (but assume no code will need to be executed)
+/// For testing only; doesn't set the filename properly!
 fn recognize(rule: &FormPat, grammar: &SynEnv, toks: &str) -> bool {
-    let (start_but_startier, chart) =
-        create_chart(Rc::new(rule.clone()), ParseContext::new_from_grammar(grammar.clone()), toks);
+    let (start_but_startier, chart) = create_chart(
+        Rc::new(rule.clone()),
+        ParseContext::new_from_grammar(grammar.clone()),
+        toks,
+        0,
+    );
 
     chart[chart.len() - 1].iter().any(|item| {
         (*item.wanted_by.borrow()).iter().any(|idr| start_but_startier.is(*idr))
@@ -299,7 +319,7 @@ impl std::fmt::Debug for Item {
     fn fmt(&self, f: &mut std::fmt::Formatter) -> std::fmt::Result {
         write!(
             f,
-            "[({:#?}){}({:#?}.{}){}<{:#?} - {:#?}>]",
+            "[({:?}){}({}.{}){}<{:?} - {:?}>]",
             self.id,
             self.start_idx,
             self.rule,
@@ -342,7 +362,7 @@ impl Item {
         use std::cmp::Ordering::*;
         let comparison = other.local_parse.borrow().partial_cmp(&*self.local_parse.borrow());
         log!(
-            "\n(For {}) Merging {:#?} and {:#?}... ",
+            "\n(For {}) Merging {:?} and {:?}... ",
             self.id.0,
             *other.local_parse.borrow(),
             *self.local_parse.borrow()
@@ -353,6 +373,9 @@ impl Item {
             }
             Some(Equal) | Some(Less) => { /* no new information */ }
             None => {
+                // TODO: maybe it would be nice to ignore ambiguities that "don't matter".
+                // But even if `local_parse` is different, the resulting `Ast` can be the same.
+                // So, maybe have `Ambiguous` store all possible ambiguities
                 let amb = LocalParse::Ambiguous(
                     Box::new(self.local_parse.borrow().clone()),
                     Box::new(other.local_parse.borrow().clone()),
@@ -405,6 +428,7 @@ impl Item {
                 done: RefCell::new(false),
                 pc: self.pc.clone(),
                 common: self.common,
+                file_id: self.file_id,
                 local_parse: RefCell::new(LocalParse::NothingYet),
                 id: get_next_id(),
                 wanted_by: Rc::new(RefCell::new(vec![self.id.get_ref()])),
@@ -434,7 +458,7 @@ impl Item {
                     //  in `shift_or_predict` for leaves.
                     // Except for `Seq`. TODO: why?
                     let mut more = match *waiting_item.rule {
-                        Anyways(_) | Impossible | Scan(_) => {
+                        Anyways(_) | Impossible | Scan(_, _) => {
                             icp!("{:#?} should not be waiting for anything!", waiting_item)
                         }
                         Seq(ref subs) => {
@@ -479,29 +503,35 @@ impl Item {
                         // Using `c_parse` instead of `local_parse` here is weird,
                         //  but probably necessary to allow `Call` under `Reserved`.
                         Reserved(_, ref name_list) => match self.c_parse(chart, cur_idx) {
-                            Ok(Ast::Atom(name)) | Ok(Ast::VariableReference(name)) => {
-                                if name_list.contains(&name) {
-                                    vec![]
-                                } else {
-                                    waiting_item.finish_with(me_justif, 0)
+                            Ok(ast_with_name) => match ast_with_name.c() {
+                                ast::Atom(name) | ast::VariableReference(name) => {
+                                    if name_list.contains(&name) {
+                                        vec![]
+                                    } else {
+                                        waiting_item.finish_with(me_justif, 0)
+                                    }
                                 }
-                            }
-                            _ => {
-                                log!("found something unusual {:?}", other);
-                                vec![]
+                                _ => {
+                                    log!("found something unusual {:?}", ast_with_name);
+                                    vec![]
+                                }
+                            },
+                            Err(_) => {
+                                log!("passing an error through `Reserved`");
+                                waiting_item.finish_with(me_justif, 0)
                             }
                         },
                         Literal(_, expected) => match self.c_parse(chart, cur_idx) {
-                            Ok(Ast::Atom(name)) => {
-                                if name == expected {
+                            Ok(ast) => {
+                                if ast.c() == &ast::Atom(expected) {
                                     waiting_item.finish_with(me_justif, 0)
                                 } else {
                                     vec![]
                                 }
                             }
-                            _ => {
-                                log!("found something unusual {:?}", other);
-                                vec![]
+                            _other => {
+                                log!("passing an error through `Literal`");
+                                waiting_item.finish_with(me_justif, 0)
                             }
                         },
                         Biased(ref _plan_a, ref plan_b) => {
@@ -525,8 +555,16 @@ impl Item {
             if let Call(_) = *res[0].0.rule {
                 // HACK: I think that `Call` is uninformative
             } else if !self.common {
-                best_token
-                    .with(|bt| *bt.borrow_mut() = (cur_idx, res[0].0.rule.clone(), res[0].0.pos));
+                // Possibly set the high-water mark for parse error purposes:
+                best_token.with(|bt| {
+                    if cur_idx > bt.borrow().0 {
+                        // We've parsed further than ever before
+                        *bt.borrow_mut() = (cur_idx, res[0].0.rule.clone(), res[0].0.pos)
+                    } else if cur_idx == bt.borrow().0 && res[0].0.pos > bt.borrow().2 {
+                        // We're deeper into a rule than ever before
+                        *bt.borrow_mut() = (cur_idx, res[0].0.rule.clone(), res[0].0.pos)
+                    }
+                });
             }
         }
 
@@ -547,14 +585,19 @@ impl Item {
             (0, &Anyways(ref a)) => self.finish_with(ParsedAtom(a.clone()), 0),
             (_, &Impossible) => vec![],
             (0, &Literal(ref sub, _)) => self.start(sub, cur_idx),
-            (0, &Scan(crate::grammar::Scanner(ref regex))) => {
+            (0, &Scan(crate::grammar::Scanner(ref regex), _)) => {
                 let mut caps = regex.capture_locations();
                 if regex.captures_read(&mut caps, &toks[cur_idx..]).is_some() {
                     match caps.get(1) {
                         Some((start, end)) => {
                             // These are byte indices!
                             self.finish_with(
-                                ParsedAtom(Ast::Atom(n(&toks[cur_idx + start..cur_idx + end]))),
+                                ParsedAtom(Ast(Rc::new(LocatedAst {
+                                    c: ast::Atom(n(&toks[cur_idx + start..cur_idx + end])),
+                                    file_id: self.file_id,
+                                    begin: cur_idx + start,
+                                    end: cur_idx + end,
+                                }))),
                                 caps.get(0).unwrap().1, // End of the *whole string consumed*
                             )
                         }
@@ -609,13 +652,28 @@ impl Item {
             (0, &SynImport(ref lhs, _, _)) => self.start(&lhs, cur_idx),
             (1, &SynImport(_, ref body, ref f)) => {
                 // TODO: handle errors properly! Probably need to memoize, also!
+                // TODO: an ambiguity or error here leads to a pretty confusing parse failure.
+                // It ought to be an outright parse error,
+                //  even if it theoretically could get papered over,
+                //  but we don't have a way to communicate those here.
                 let partial_parse = match *self.local_parse.borrow() {
-                    NothingYet | Ambiguous(_, _) => return vec![],
+                    NothingYet => return vec![],
+                    Ambiguous(_, _) => {
+                        println!(
+                            "Warning: ambiguity in syntax import! {:?}",
+                            *self.local_parse.borrow()
+                        );
+                        return vec![];
+                    }
                     ParsedAtom(ref a) => a.clone(),
                     JustifiedByItem(_) | JustifiedByItemPlanB(_) => {
-                        match self.find_wanted(chart, cur_idx).c_parse(chart, cur_idx) {
-                            Ok(ast) => ast,
-                            Err(_) => {
+                        match self
+                            .find_wanted(chart, cur_idx)
+                            .map(|item| item.c_parse(chart, cur_idx))
+                        {
+                            Ok(Ok(ast)) => ast,
+                            e => {
+                                println!("Warning: error in syntax import! {:?}", e);
                                 return vec![];
                             }
                         }
@@ -646,6 +704,7 @@ impl Item {
                         done: RefCell::new(false),
                         pc: Rc::new(new_ctxt),
                         common: false,
+                        file_id: self.file_id,
                         local_parse: RefCell::new(LocalParse::NothingYet),
                         id: get_next_id(),
                         wanted_by: Rc::new(RefCell::new(vec![self.id.get_ref()])),
@@ -671,7 +730,11 @@ impl Item {
         }
     }
 
-    fn find_wanted<'f, 'c>(&'f self, chart: &'c [Vec<Item>], done_tok: usize) -> &'c Item {
+    fn find_wanted<'f, 'c>(
+        &'f self,
+        chart: &'c [Vec<Item>],
+        done_tok: usize,
+    ) -> Result<&'c Item, ParseError> {
         let mut first_found: Option<&Item> = None;
         let local_parse = self.local_parse.borrow().clone();
         let desired_id = match local_parse {
@@ -686,7 +749,9 @@ impl Item {
                 let l_res = self.c_parse(chart, done_tok).unwrap();
                 *self.local_parse.borrow_mut() = r;
                 let r_res = self.c_parse(chart, done_tok).unwrap();
-                panic!("Ambiguity! \n=L=>{}\n=R=>{}\n", l_res, r_res)
+                return Err(ParseError {
+                    msg: format!("Ambiguity! \n=L=>{}\n=R=>{}\n", l_res, r_res),
+                });
             }
             _ => icp!("tried to parse unjustified item: {:#?} ", self),
         };
@@ -705,7 +770,17 @@ impl Item {
             }
         }
 
-        first_found.expect("ICP: no parse after successful recognition")
+        Ok(first_found.expect("ICP: no parse after successful recognition"))
+    }
+
+    /// Put location information on an `AstContents`, forming an `Ast`.
+    fn locate(&self, done_tok: usize, c: ast::AstContents) -> Ast {
+        Ast(Rc::new(ast::LocatedAst {
+            c: c,
+            file_id: self.file_id,
+            begin: self.start_idx,
+            end: done_tok,
+        }))
     }
 
     /// After the chart is built, we parse...
@@ -715,17 +790,20 @@ impl Item {
         let res = match *self.rule {
             Anyways(ref a) => Ok(a.clone()),
             Impossible => icp!("Parser parsed the impossible!"),
-            Scan(_) => match self.local_parse.borrow().clone() {
+            Scan(_, _) => match self.local_parse.borrow().clone() {
                 ParsedAtom(a) => Ok(a),
-                NothingYet => Ok(Ast::Trivial),
+                NothingYet => Ok(ast!((trivial))), // TODO: should we fake location info here?
                 _ => icp!(),
             },
-            VarRef(_) => match self.find_wanted(chart, done_tok).c_parse(chart, done_tok)? {
-                Ast::Atom(a) => Ok(Ast::VariableReference(a)),
-                _ => icp!("no atom saved"),
-            },
+            VarRef(_) => {
+                let var_ast = self.find_wanted(chart, done_tok)?.c_parse(chart, done_tok)?;
+                match var_ast.c() {
+                    ast::Atom(a) => Ok(var_ast.with_c(ast::VariableReference(*a))),
+                    _ => icp!("no atom saved"),
+                }
+            }
             Literal(_, _) | Alt(_) | Biased(_, _) | Call(_) | Reserved(_, _) | Common(_) => {
-                self.find_wanted(chart, done_tok).c_parse(chart, done_tok)
+                self.find_wanted(chart, done_tok)?.c_parse(chart, done_tok)
             }
             Seq(_) | Star(_) | Plus(_) | SynImport(_, _, _) => {
                 let mut step = self;
@@ -743,7 +821,7 @@ impl Item {
                         break;
                     }
 
-                    let sub = step.find_wanted(chart, pos);
+                    let sub = step.find_wanted(chart, pos)?;
                     subtrees.push(sub.c_parse(chart, pos)?);
                     if sub.start_idx == self.start_idx && step.pos == 1 {
                         break;
@@ -769,25 +847,28 @@ impl Item {
                 }
                 subtrees.reverse();
 
-                match *self.rule {
-                    Seq(_) | SynImport(_, _, _) => Ok(Ast::Shape(subtrees)),
-                    Star(_) | Plus(_) => Ok(Ast::IncompleteNode(EnvMBE::new_from_anon_repeat(
+                Ok(self.locate(done_tok, match *self.rule {
+                    Seq(_) | SynImport(_, _, _) => ast::Shape(subtrees),
+                    Star(_) | Plus(_) => ast::IncompleteNode(EnvMBE::new_from_anon_repeat(
                         subtrees.into_iter().map(|a| a.flatten()).collect(),
-                    ))),
+                    )),
                     _ => icp!("seriously, this can't happen"),
-                }
+                }))
             }
             Named(name, _) => {
-                let sub_parsed = self.find_wanted(chart, done_tok).c_parse(chart, done_tok)?;
-                Ok(Ast::IncompleteNode(EnvMBE::new_from_leaves(Assoc::single(name, sub_parsed))))
+                let sub_parsed = self.find_wanted(chart, done_tok)?.c_parse(chart, done_tok)?;
+                Ok(sub_parsed.with_c(ast::IncompleteNode(EnvMBE::new_from_leaves(Assoc::single(
+                    name,
+                    sub_parsed.clone(),
+                )))))
             }
             Scope(ref form, ref export) => {
-                let sub_parsed = self.find_wanted(chart, done_tok).c_parse(chart, done_tok)?;
+                let sub_parsed = self.find_wanted(chart, done_tok)?.c_parse(chart, done_tok)?;
                 // TODO #14: We should add zero-length repeats of missing `Named`s,
-                Ok(Ast::Node(form.clone(), sub_parsed.flatten(), export.clone()))
+                Ok(sub_parsed.with_c(ast::Node(form.clone(), sub_parsed.flatten(), export.clone())))
             }
             Pick(_, name) => {
-                let sub_parsed = self.find_wanted(chart, done_tok).c_parse(chart, done_tok)?;
+                let sub_parsed = self.find_wanted(chart, done_tok)?.c_parse(chart, done_tok)?;
                 sub_parsed
                     .flatten()
                     .get_leaf(name)
@@ -797,23 +878,23 @@ impl Item {
                     .map(std::clone::Clone::clone)
             }
             NameImport(_, ref beta) => {
-                let sub_parsed = self.find_wanted(chart, done_tok).c_parse(chart, done_tok)?;
-                Ok(Ast::ExtendEnv(Box::new(sub_parsed), beta.clone()))
+                let sub_parsed = self.find_wanted(chart, done_tok)?.c_parse(chart, done_tok)?;
+                Ok(sub_parsed.with_c(ast::ExtendEnv(sub_parsed.clone(), beta.clone())))
             }
             NameImportPhaseless(_, ref beta) => {
-                let sub_parsed = self.find_wanted(chart, done_tok).c_parse(chart, done_tok)?;
-                Ok(Ast::ExtendEnvPhaseless(Box::new(sub_parsed), beta.clone()))
+                let sub_parsed = self.find_wanted(chart, done_tok)?.c_parse(chart, done_tok)?;
+                Ok(sub_parsed.with_c(ast::ExtendEnvPhaseless(sub_parsed.clone(), beta.clone())))
             }
             QuoteDeepen(_, pos) => {
-                let sub_parsed = self.find_wanted(chart, done_tok).c_parse(chart, done_tok)?;
-                Ok(Ast::QuoteMore(Box::new(sub_parsed), pos))
+                let sub_parsed = self.find_wanted(chart, done_tok)?.c_parse(chart, done_tok)?;
+                Ok(sub_parsed.with_c(ast::QuoteMore(sub_parsed.clone(), pos)))
             }
             QuoteEscape(_, depth) => {
-                let sub_parsed = self.find_wanted(chart, done_tok).c_parse(chart, done_tok)?;
-                Ok(Ast::QuoteLess(Box::new(sub_parsed), depth))
+                let sub_parsed = self.find_wanted(chart, done_tok)?.c_parse(chart, done_tok)?;
+                Ok(sub_parsed.with_c(ast::QuoteLess(sub_parsed.clone(), depth)))
             }
         };
-        log!(">>>{:#?}<<<\n", res);
+        log!(">>>{:?}<<<\n", res);
 
         res
     }
@@ -827,15 +908,22 @@ pub struct ParseError {
 }
 
 pub fn parse(rule: &FormPat, pc: ParseContext, toks: &str) -> ParseResult {
-    best_token.with(|bt| *bt.borrow_mut() = (0, Rc::new(rule.clone()), 0));
+    // `parse` gets called recursively, so stash our mutable state:
+    let orig_best_token = best_token.with(|bt| {
+        let orig = bt.borrow().clone();
+        *bt.borrow_mut() = (0, Rc::new(rule.clone()), 0);
+        orig
+    });
 
-    let (start_but_startier, chart) = create_chart(Rc::new(rule.clone()), pc, toks);
+    let file_id = files.with(|f| f.borrow_mut().add("[filename]".to_string(), toks.to_string()));
+
+    let (start_but_startier, chart) = create_chart(Rc::new(rule.clone()), pc, toks, file_id);
     let final_item = chart[chart.len() - 1].iter().find(|item| {
         (*item.wanted_by.borrow()).iter().any(|idr| start_but_startier.is(*idr))
             && *item.done.borrow()
     });
-    log!("-------\n");
-    match final_item {
+    log!("\n-------\n");
+    let result = match final_item {
         Some(i) => i.c_parse(&chart, chart.len() - 1),
         None => best_token.with(|bt| {
             let (idx, ref grammar, pos) = *bt.borrow();
@@ -846,16 +934,19 @@ pub fn parse(rule: &FormPat, pc: ParseContext, toks: &str) -> ParseResult {
 
             Err(ParseError {
                 msg: format!(
-                    "Could not parse past “{}•{}” (on line {}) \nin rule {:?} at {}",
+                    "Could not parse past “{}•{}” (on line {}) \nin rule {}",
                     &toks[line_begin..idx],
                     &toks[idx..line_end],
                     line_number,
-                    grammar,
-                    pos
+                    grammar.mark_up(Some(pos)),
                 ),
             })
         }),
-    }
+    };
+
+    best_token.with(|bt| *bt.borrow_mut() = orig_best_token); // unstash
+
+    result
 }
 
 pub fn parse_in_syn_env(rule: &FormPat, syn_env: SynEnv, toks: &str) -> ParseResult {
@@ -868,7 +959,7 @@ fn parse_top(rule: &FormPat, toks: &str) -> ParseResult {
 
 #[test]
 fn earley_merging() {
-    let one_rule = crate::grammar::new_scan("whatever");
+    let one_rule = crate::grammar::new_scan("whatever", None);
     let another_rule = Impossible;
     let main_grammar = assoc_n!("a" => Rc::new(form_pat!((scan "irrelevant"))));
     let another_grammar = assoc_n!("a" => Rc::new(form_pat!((scan "irrelevant"))));
@@ -880,6 +971,7 @@ fn earley_merging() {
         pos: 0,
         pc: Rc::new(ParseContext::new_from_grammar(main_grammar.clone())),
         common: false,
+        file_id: 0,
         id: get_next_id(),
         done: RefCell::new(false),
         local_parse: RefCell::new(LocalParse::NothingYet),
@@ -998,13 +1090,13 @@ fn earley_merging() {
 fn earley_simple_recognition() {
     let main_grammar = Assoc::new();
 
-    let atom = Rc::new(crate::grammar::new_scan(r"\s*(\S+)"));
+    let atom = Rc::new(crate::grammar::new_scan(r"\s*(\S+)", None));
 
     // 0-length strings
 
     assert_eq!(recognize(&*atom, &main_grammar, tokens_s!()), false);
 
-    assert_eq!(recognize(&Anyways(Ast::Trivial), &main_grammar, tokens_s!()), true);
+    assert_eq!(recognize(&Anyways(ast!((trivial))), &main_grammar, tokens_s!()), true);
 
     assert_eq!(recognize(&Seq(vec![]), &main_grammar, tokens_s!()), true);
 
@@ -1145,7 +1237,7 @@ fn earley_simple_recognition() {
 #[test]
 fn earley_env_recognition() {
     fn mk_lt(s: &str) -> Rc<FormPat> {
-        Rc::new(Literal(Rc::new(crate::grammar::new_scan(r"\s*(\S+)")), n(s)))
+        Rc::new(Literal(Rc::new(crate::grammar::new_scan(r"\s*(\S+)", None)), n(s)))
     }
     let env = assoc_n!(
         "empty" => Rc::new(Seq(vec![])),
@@ -1217,10 +1309,10 @@ fn earley_env_recognition() {
 #[test]
 fn basic_parsing_e() {
     fn mk_lt(s: &str) -> Rc<FormPat> {
-        Rc::new(Literal(Rc::new(crate::grammar::new_scan(r"\s*(\S+)")), n(s)))
+        Rc::new(Literal(Rc::new(crate::grammar::new_scan(r"\s*(\S+)", None)), n(s)))
     }
 
-    let atom = Rc::new(crate::grammar::new_scan(r"\s*(\S+)"));
+    let atom = Rc::new(crate::grammar::new_scan(r"\s*(\S+)", None));
 
     assert_eq!(parse_top(&*atom, tokens_s!("asdf")), Ok(ast!("asdf")));
 
@@ -1331,3 +1423,24 @@ fn basic_parsing_e() {
         Ok(ast!("Moon"))
     );
 }
+
+#[test]
+fn ambiguity() {
+    let no_env = Assoc::new();
+    // TODO: prevent an infinite loop in this case:
+    let _star_star = form_pat![(star (scan "a*"))];
+
+    let a_vs_a = form_pat![(alt (scan "a"), (scan "a"))];
+
+    assert_eq!(recognize(&a_vs_a, &no_env, "a"), true);
+    assert_m!(parse_top(&a_vs_a, ""), Err(_));
+
+    let a_vs_aa__twice = form_pat![[(alt (scan "a"), (scan "aa")),
+                                    (alt (scan "a"), (scan "aa"))]];
+
+    assert_eq!(recognize(&a_vs_aa__twice, &no_env, "aaa"), true);
+    assert_m!(parse_top(&a_vs_aa__twice, "aaa"), Err(_));
+
+    assert_eq!(recognize(&a_vs_aa__twice, &no_env, "aaaa"), true);
+    assert_m!(parse_top(&a_vs_aa__twice, "aaaa"), Ok(_)); // Shape of trivials. Sure.
+}
