@@ -26,8 +26,8 @@ custom_derive! {
 
         /// Matches actual text!
         /// The regex must have a single capturing group.
-        /// The contents of the capturing group are
-        Scan(Scanner),
+        /// The optional string is a TextMate syntax category.
+        Scan(Scanner, Option<String>),
 
         /// Marks this rule as too commonly-used to be informative;
         ///  prevents display of this rule in parse errors,
@@ -79,7 +79,69 @@ custom_derive! {
     }
 }
 
+impl std::fmt::Display for FormPat {
+    fn fmt(&self, f: &mut std::fmt::Formatter) -> Result<(), std::fmt::Error> {
+        write!(f, "{}", self.mark_up(None))
+    }
+}
+
 impl FormPat {
+    pub fn mark_up(&self, pos: Option<usize>) -> String {
+        match self {
+            Anyways(ref ast) => {
+                let contents = format!("{}", ast);
+                if contents.len() > 20 {
+                    format!("anyways{{⋯}}anyways")
+                } else {
+                    format!("anyways{{ {} }}anyways", contents)
+                }
+            }
+            Impossible => format!("impossible"),
+            Scan(ref scanner, _) => format!("/{}/", scanner.0),
+            Common(ref body) => format!("common ({})", body),
+            Reserved(ref body, ref names) => format!(
+                "{} reserving {}",
+                body,
+                names.iter().map(|name| format!("'{}'", name)).collect::<Vec<_>>().join(" ")
+            ),
+            Literal(ref _body, name) => format!("\x1b[34m{}\x1b[0m", name),
+            VarRef(ref body) => format!("vr ({})", body),
+            Seq(ref elts) => {
+                let mut formatted_elts = elts.iter().map(|e| format!("{}", e)).collect::<Vec<_>>();
+                if let Some(pos) = pos {
+                    formatted_elts.insert(pos, "•".to_string());
+                }
+                format!("[{}]", formatted_elts.join(" "))
+            }
+            Star(ref body) => format!("{} *", body),
+            Plus(ref body) => format!("{} +", body),
+            Alt(ref elts) => format!(
+                "alt[{}]alt",
+                elts.iter().map(|e| format!("{}", e)).collect::<Vec<_>>().join(" ")
+            ),
+            Biased(ref plan_a, ref plan_b) => format!("{} or{{{}}}or", plan_a, plan_b),
+            // In macro definitions, we need a type here, but it's not present in the grammar:
+            Call(ref name) => format!(",{{{}}},", name),
+            SynImport(ref def, ref body, ref _synex) => match pos {
+                None => format!("??syntax extension {} => {} ??", def, body),
+                Some(0) => format!("??syntax extension • {} => {} ??", def, body),
+                Some(1) => format!("??syntax extension {} => • {} ??", def, body),
+                Some(2) => format!("??syntax extension {} => {} • ??", def, body),
+                _ => icp!(),
+            },
+
+            Scope(ref form, ref ebeta) => {
+                format!("'{{{}}}' ??? -> .{{??}}. => {:?}", form.grammar, ebeta)
+            }
+            Named(name, ref body) => format!("{} := ({})", name, body),
+            Pick(ref body, name) => format!("pick {} in {}", name, body),
+            NameImport(ref body, ref beta) => format!("{} <-- {:?}", body, beta),
+            NameImportPhaseless(ref body, ref beta) => format!("{} <--?? {:?}", body, beta),
+            QuoteDeepen(ref body, pos) => format!("??deepen?? {} {}", body, pos),
+            QuoteEscape(ref body, levels) => format!("??escape?? {} {}", body, levels),
+        }
+    }
+
     // Finds all `Named` nodes, and how many layers of repetition they are underneath.
     pub fn binders(&self) -> Vec<(Name, u8)> {
         use tap::tap::Tap;
@@ -107,7 +169,7 @@ impl FormPat {
             Biased(ref body_a, ref body_b) => {
                 body_a.binders().tap_mut(|v| v.append(&mut body_b.binders()))
             }
-            Anyways(_) | Impossible | Literal(_, _) | Scan(_) | VarRef(_) | Call(_) => vec![],
+            Anyways(_) | Impossible | Literal(_, _) | Scan(_, _) | VarRef(_) | Call(_) => vec![],
         }
     }
 
@@ -129,7 +191,7 @@ impl FormPat {
             Call(_) => None,
             // TODO: `Pick` should return a result.
             Scope(_, _) | Pick(_, _) => None, // Only look in the current scope
-            Anyways(_) | Impossible | Scan(_) => None,
+            Anyways(_) | Impossible | Scan(_, _) => None,
             Star(ref body)
             | Plus(ref body)
             | SynImport(ref body, _, _)
@@ -155,6 +217,63 @@ impl FormPat {
             }
         }
     }
+
+    /// Map regular expressions (as strings) to their TextMate categories.
+    /// This is a loose approximation of the impossible task of syntax-highlighting a `FormPat`.
+    pub fn textmate_categories(&self) -> Vec<(String, String)> {
+        match self {
+            Scan(scanner, name) => match name {
+                Some(name) => vec![(
+                    format!("{}", scanner.0)
+                        .strip_prefix("^")
+                        .unwrap()
+                        .replace(r#"\/"#, r#"/"#)
+                        .to_string(),
+                    name.clone(),
+                )],
+                None => vec![],
+            },
+            Reserved(body, reserved) => {
+                // TODO: modify `Reserved` to allow customization
+                let mut res = body.textmate_categories();
+                let matcher = reserved
+                    .iter()
+                    .map(|word| regex::escape(&word.orig_sp()))
+                    .collect::<Vec<_>>()
+                    .join("|");
+                res.push((matcher, "keyword".to_string()));
+                res
+            }
+            Literal(_, lit) => {
+                vec![(regex::escape(&lit.orig_sp()), "keyword.operator".to_string())]
+            }
+            Scope(form, _) => form.grammar.textmate_categories(),
+            Anyways(_) | Impossible | Call(_) => vec![],
+            Common(body)
+            | VarRef(body)
+            | Star(body)
+            | Plus(body)
+            | SynImport(_, body, _)
+            | Named(_, body)
+            | Pick(body, _)
+            | NameImport(body, _)
+            | NameImportPhaseless(body, _)
+            | QuoteDeepen(body, _)
+            | QuoteEscape(body, _) => body.textmate_categories(),
+            Biased(lhs, rhs) => {
+                let mut res = lhs.textmate_categories();
+                res.append(&mut rhs.textmate_categories());
+                res
+            }
+            Seq(bodies) | Alt(bodies) => {
+                let mut res = vec![];
+                for body in bodies {
+                    res.append(&mut body.textmate_categories());
+                }
+                res
+            }
+        }
+    }
 }
 
 #[derive(Clone)]
@@ -185,8 +304,8 @@ impl std::fmt::Debug for SyntaxExtension {
     }
 }
 
-pub fn new_scan(regex: &str) -> FormPat {
-    Scan(Scanner(regex::Regex::new(&format!("^{}", regex)).unwrap()))
+pub fn new_scan(regex: &str, cat: Option<String>) -> FormPat {
+    Scan(Scanner(regex::Regex::new(&format!("^{}", regex)).unwrap()), cat)
 }
 
 #[derive(Clone)]
@@ -226,8 +345,10 @@ use self::FormPat::*;
 
 #[test]
 fn basic_parsing() {
-    fn mk_lt(s: &str) -> Rc<FormPat> { Rc::new(Literal(Rc::new(new_scan(r"\s*(\S+)")), n(s))) }
-    let atom = Rc::new(new_scan(r"\s*(\S+)"));
+    fn mk_lt(s: &str) -> Rc<FormPat> {
+        Rc::new(Literal(Rc::new(new_scan(r"\s*(\S+)", None)), n(s)))
+    }
+    let atom = Rc::new(new_scan(r"\s*(\S+)", None));
 
     assert_eq!(parse_top(&Seq(vec![atom.clone()]), tokens_s!("asdf")).unwrap(), ast_shape!("asdf"));
 
@@ -385,7 +506,7 @@ fn advanced_parsing() {
 #[test]
 fn extensible_parsing() {
     use crate::{
-        ast::Ast::IncompleteNode,
+        ast::AstContents::IncompleteNode,
         earley::{parse_in_syn_env, ParseContext},
     };
 
@@ -444,7 +565,7 @@ fn extensible_parsing() {
     );
 
     fn counter_synex(_: ParseContext, a: Ast) -> ParseContext {
-        let count = match a {
+        let count = match a.c() {
             IncompleteNode(mbe) => mbe,
             _ => panic!(),
         }
@@ -452,7 +573,7 @@ fn extensible_parsing() {
         .len();
 
         ParseContext::new_from_grammar(
-            assoc_n!("count" => Rc::new(Literal(Rc::new(new_scan(r"\s*(\S+)")),
+            assoc_n!("count" => Rc::new(Literal(Rc::new(new_scan(r"\s*(\S+)", None)),
                                             n(&count.to_string())))),
         )
     }
