@@ -43,7 +43,7 @@ macro_rules! beta_connector {
     ($lhs:tt = $rhs:tt) => {
         crate::beta::SameAs(
             crate::name::n(expr_ify!($lhs)),
-            Box::new(crate::ast::VariableReference(crate::name::n(expr_ify!($rhs)))),
+            Box::new(raw_ast!(VariableReference(crate::name::n(expr_ify!($rhs))))),
         )
     };
     // TODO: this needs a better notation, somehow
@@ -147,23 +147,42 @@ macro_rules! t_elt {
 
 // Ast
 
+// AstContents to Ast.
+// Whenever possible, use `ast!` directly (or, in tests, `u!`).
+macro_rules! raw_ast {
+    ($ast_kind:ident) => {
+        crate::ast::Ast(std::rc::Rc::new(crate::ast::LocatedAst {
+            c: crate::ast::AstContents::$ast_kind,
+            // TODO: would Rust file info be useful?
+            file_id: 0, begin: 0, end: 0
+        }))
+    };
+    ($ast_kind:ident ( $( $body:expr ),* ) ) => {
+        crate::ast::Ast(std::rc::Rc::new(crate::ast::LocatedAst {
+            c: crate::ast::AstContents::$ast_kind( $( $body ),* ),
+            // TODO: would Rust file info be useful?
+            file_id: 0, begin: 0, end: 0
+        }))
+    }
+}
+
 macro_rules! ast_shape {
-    ($($contents:tt)*) => { crate::ast::Shape(vec![ $(  ast!($contents) ),* ] )};
+    ($($contents:tt)*) => { raw_ast!(Shape(vec![ $(  ast!($contents) ),* ] ))};
 }
 
 macro_rules! ast {
-    ( (trivial) ) => { crate::ast::Trivial };
+    ( (trivial) ) => { raw_ast!(Trivial) };
     ( (++ $pos:tt $sub:tt) ) => {
-        crate::ast::QuoteMore(Box::new(ast!($sub)), $pos)
+        raw_ast!(QuoteMore(ast!($sub), $pos))
     };
     ( (-- $depth:tt $sub:tt ) ) => {
-        crate::ast::QuoteLess(Box::new(ast!($sub)), $depth)
+        raw_ast!(QuoteLess(ast!($sub), $depth))
     };
     ( (import $beta:tt $sub:tt) ) => {
-        crate::ast::ExtendEnv(Box::new(ast!($sub)), beta!($beta))
+        raw_ast!(ExtendEnv(ast!($sub), beta!($beta)))
     };
     ( (import_phaseless $beta:tt $sub:tt) ) => {
-        crate::ast::ExtendEnvPhaseless(Box::new(ast!($sub)), beta!($beta))
+        raw_ast!(ExtendEnvPhaseless(ast!($sub), beta!($beta)))
     };
     /* // not sure we'll need this
     ( (* $env:expr => $new_env:ident / $($n:expr),* ; $($sub_ar"gs:tt)*) ) => {
@@ -177,31 +196,34 @@ macro_rules! ast {
             Shape(res)
         }
     };*/
-    ( (vr $var:expr) ) => { crate::ast::VariableReference(crate::name::n($var)) };
+    ( (vr $var:expr) ) => { raw_ast!(VariableReference(crate::name::Name::from($var))) };
+    // Like the last clause, but explicit about being an atom:
+    ( (at $var:expr) ) => { raw_ast!(Atom(crate::name::Name::from($var))) };
     ( (, $interpolate:expr)) => { $interpolate };
     // TODO: maybe we should use commas for consistency:
-    ( ( $( $list:tt )* ) ) => { ast_shape!($($list)*)};
+    ( ( $( $list:tt )* ) ) => { ast_shape!( $($list)* ) };
     ( { - $($mbe_arg:tt)* } ) => {
-        crate::ast::IncompleteNode(mbe!( $($mbe_arg)* ))
+        raw_ast!(IncompleteNode(mbe!( $($mbe_arg)* )))
     };
     ( { $nt:tt $form:tt => $beta:tt : $($mbe_arg:tt)*} ) => {
-        crate::ast::Node(crate::core_forms::find($nt, $form), mbe!( $($mbe_arg)* ),
-                    ebeta!($beta))
+        raw_ast!(Node(crate::core_forms::find($nt, $form), mbe!( $($mbe_arg)* ),
+                    ebeta!($beta)))
     };
     ( { $form:expr => $beta:tt ; $($mbe_arg:tt)*} ) => {
-        crate::ast::Node($form, mbe!( $($mbe_arg)* ), ebeta!($beta))
+        raw_ast!(Node($form, mbe!( $($mbe_arg)* ), ebeta!($beta)))
     };
     ( { $form:expr; [ $($mbe_arg:tt)* ] }) => {
         ast!( { $form ; $($mbe_arg)* } )
     };
     ( { $form:expr; $($mbe_arg:tt)* }) => {
-        crate::ast::Node($form, mbe!( $($mbe_arg)* ), crate::beta::ExportBeta::Nothing)
+        raw_ast!(Node($form, mbe!( $($mbe_arg)* ), crate::beta::ExportBeta::Nothing))
     };
     ( { $nt:tt $form:tt : $($mbe_arg:tt)* }) => {
-        crate::ast::Node(crate::core_forms::find($nt, $form), mbe!( $($mbe_arg)* ),
-                    crate::beta::ExportBeta::Nothing)
+        raw_ast!(Node(crate::core_forms::find($nt, $form), mbe!( $($mbe_arg)* ),
+                    crate::beta::ExportBeta::Nothing))
     };
-    ($e:expr) => { crate::ast::Atom(crate::name::n($e))}
+    // Accepts either `&str` (most common) or `Names`:
+    ($e:expr) => { raw_ast!(Atom(crate::name::Name::from($e))) };
 }
 
 // These construct spanned type errors (so, for type synthesis, not subtyping)
@@ -214,7 +236,7 @@ macro_rules! ty_err_val {
 
 macro_rules! ty_err {
     ( $name:tt ( $($arg:expr),* ) at $loc:expr) => {
-        return Err(ty_err_val!( $name ( $($arg),* ) at $loc));
+        return Err(ty_err_val!( $name ( $($arg),* ) at $loc))
     }
 }
 
@@ -328,16 +350,25 @@ macro_rules! form_pat {
             crate::name::n($e))
     };
     ((lit_aat $e:expr)) => {
-        crate::grammar::FormPat::Literal(std::rc::Rc::new(crate::grammar::new_scan(r"\s*(\S+)")),
-                                    crate::name::n($e))
+        crate::grammar::FormPat::Literal(
+            std::rc::Rc::new(crate::grammar::new_scan(r"\s*(\S+)", None)),
+            crate::name::n($e))
     };
     ((name_lit__by_name $e:expr)) => {
         crate::grammar::FormPat::Literal(
             std::rc::Rc::new(crate::grammar::FormPat::Call(crate::name::n("DefaultWord"))),
             $e)
     };
+    ((lit_tok $tok:tt, $lit:expr)) => {
+        crate::grammar::FormPat::Literal(
+            std::rc::Rc::new(form_pat!($tok)),
+            crate::name::n($lit))
+    };
+    ((scan_cat $e:expr, $cat:expr)) => {
+        crate::grammar::new_scan($e, Some(String::from($cat)))
+    };
     ((scan $e:expr)) => {
-        crate::grammar::new_scan($e)
+        crate::grammar::new_scan($e, None)
     };
     ((reserved $body:tt, $( $res:tt )*)) => {
         crate::grammar::FormPat::Reserved(std::rc::Rc::new(form_pat!($body)), vec![$( n($res) ),*])
@@ -358,19 +389,19 @@ macro_rules! form_pat {
         std::rc::Rc::new(crate::grammar::FormPat::Call(crate::name::n($n)))
     ) };
     (varref_aat) => { crate::grammar::FormPat::VarRef(
-        std::rc::Rc::new(crate::grammar::new_scan(r"\s*(\S+)"))
+        std::rc::Rc::new(crate::grammar::new_scan(r"\s*(\S+)", None))
     ) };
     ((delim $n:expr, $d:expr, $body:tt)) => {
         crate::grammar::FormPat::Seq(vec![
             std::rc::Rc::new(crate::grammar::FormPat::Literal(
-                std::rc::Rc::new(crate::grammar::FormPat::Call(crate::name::n("DefaultToken"))),
+                std::rc::Rc::new(crate::grammar::FormPat::Call(crate::name::n("OpenDelim"))),
                 crate::name::n($n))),
             std::rc::Rc::new(form_pat!($body)),
             {
                 let mut main_tok = $n.to_owned();
                 main_tok.pop();
                 std::rc::Rc::new(crate::grammar::FormPat::Literal(
-                    std::rc::Rc::new(crate::grammar::FormPat::Call(crate::name::n("DefaultToken"))),
+                    std::rc::Rc::new(crate::grammar::FormPat::Call(crate::name::n("CloseDelim"))),
                     crate::name::n(&format!("{}{}", crate::read::delim($d).close(), main_tok))))
             }])
     };
@@ -488,7 +519,8 @@ macro_rules! negative_typed_form {
 }
 
 // Value
-
+// (This is exported so `cli.rs` can use it. TODO: bring those tests back into the library.)
+#[macro_export]
 macro_rules! val {
     (i $i:expr) => { crate::runtime::eval::Value::Int(::num::bigint::BigInt::from($i)) };
     (b $b:expr) => {
@@ -584,6 +616,7 @@ macro_rules! core_fn {
 }
 
 // Alpha
+#[macro_export]
 macro_rules! without_freshening {
     ($( $body:tt )*) => {{
         let mut orig: bool = false;
@@ -604,7 +637,7 @@ macro_rules! without_freshening {
 macro_rules! expect_node {
     ( ($node:expr ; $form:expr) $env:ident ; $body:expr ) => {
         // This is tied to the signature of `Custom`
-        if let Node(ref f, ref $env, _) = $node {
+        if let crate::ast::Node(ref f, ref $env, _) = $node.c() {
             if *f == $form {
                 $body
             } else {
@@ -656,6 +689,7 @@ macro_rules! node_let {
     )
 }
 
+// TODO: get rid of these; just use `form_pat!((alt …))` directly.
 macro_rules! forms_to_form_pat {
     ( $( $form:expr ),* ) => {
         form_pat!((alt $( (scope $form) ),* ))
@@ -722,7 +756,7 @@ macro_rules! vrep_accum {
 }
 
 // Testing
-
+#[macro_export]
 macro_rules! assert_m {
     ($got:expr, $expected:pat => $body:stmt) => {{
         let got = $got;
