@@ -187,6 +187,36 @@ impl<T: Clone + fmt::Debug> fmt::Debug for EnvMBE<T> {
     }
 }
 
+impl<T: Clone + fmt::Display> fmt::Display for EnvMBE<T> {
+    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
+        if self.leaves.empty() && self.repeats.is_empty() {
+            write!(f, "∅")
+        } else {
+            write!(f, "{{MBE {}, ✶", self.leaves)?;
+            let mut first = true;
+            for (i, rep) in self.repeats.iter().enumerate() {
+                if !first {
+                    write!(f, ", ")?;
+                }
+                first = false;
+
+                // is it a named repeat?
+                for (name, idx_maybe) in self.named_repeats.iter_pairs() {
+                    if let Some(idx) = *idx_maybe {
+                        if idx == i {
+                            write!(f, "⋯({})⋯ ", name)?;
+                        }
+                    }
+                }
+                for mbe in &**rep {
+                    write!(f, "{} ", mbe)?;
+                }
+            }
+            write!(f, "}}")
+        }
+    }
+}
+
 impl<T: Clone> EnvMBE<T> {
     pub fn new() -> EnvMBE<T> {
         EnvMBE {
@@ -416,8 +446,16 @@ impl<T: Clone> EnvMBE<T> {
                 .iter()
                 .enumerate()
                 .map(|(rpt_idx, rc_vec_mbe): (usize, &Rc<Vec<EnvMBE<T>>>)| {
-                    let this_rpt_name = self.leaf_locations.find_value(&Some(rpt_idx)).unwrap();
-                    let marched_ctxts = ctxt.march_all(&[*this_rpt_name]);
+                    let marched_ctxts = match self.leaf_locations.find_value(&Some(rpt_idx)) {
+                        Some(this_rpt_name) => ctxt.march_all(&[*this_rpt_name]),
+                        None => {
+                            // This repeat has no leaves.
+                            let mut res = vec![];
+                            res.resize(rc_vec_mbe.len(), ctxt.clone());
+                            res
+                        }
+                    };
+
                     Rc::new(
                         rc_vec_mbe
                             .iter()
